# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SlugRedirect',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('old_object_slug', models.CharField(max_length=200)),
                ('new_object_id', models.PositiveIntegerField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='slugredirect',
            unique_together=set([('content_type', 'old_object_slug')]),
        ),
    ]



from django.contrib.contenttypes.models import ContentType
from django.shortcuts import redirect

from .models import SlugRedirect

def get_slug_redirect(model, slug):
    try:
        sr = SlugRedirect.objects.get(
            content_type=ContentType.objects.get_for_model(model),
            old_object_slug=slug
        )
        return sr.new_object
    except SlugRedirect.DoesNotExist:
        return None


class SlugRedirectMixin(object):
    """Adds SlugRedirect redirection for DetailView-derived classes"""

    def redirect_to(self, correct_object):
        return redirect(correct_object)

    def get(self, request, *args, **kwargs):
        # Check if this is an old slug for redirection:
        slug = kwargs['slug']
        object_to_redirect_to = get_slug_redirect(self.model, slug)
        if object_to_redirect_to:
            return self.redirect_to(object_to_redirect_to)
        # Otherwise look up the slug as normal:
        else:
            return super(SlugRedirectMixin, self).get(request, *args, **kwargs)



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import models
from django.apps import apps

# This is based on
# https://github.com/dracos/Theatricalia/blob/master/merged/models.py
# but adapted for redirecting from an old slug rather than ID.

class SlugRedirect(models.Model):
    """A model to represent a redirect from an old slug

    This is particular useful if you're merging two records, but you
    don't want the old URL to break.  It's also useful if you need to
    change a slug.
    """

    content_type = models.ForeignKey(ContentType)
    old_object_slug = models.CharField(max_length=200)
    new_object_id = models.PositiveIntegerField()
    new_object = GenericForeignKey('content_type', 'new_object_id')

    created = models.DateTimeField( auto_now_add=True )
    updated = models.DateTimeField( auto_now=True )

    def __unicode__(self):
        return u'slug "%s" -> %s' % (self.old_object_slug, self.new_object)

    class Meta:
        unique_together = ("content_type", "old_object_slug")

# This is a validator you can use on any model's slug field to check
# that its slug isn't already being redirected. To use it you should
# use functools.partial to supply the model class.

def validate_slug_not_redirecting(app_label, model_name, slug):
    """A function for building validator functions for any model's slug field

    You can use this to build a validator for any model's slug field
    to check that you're not trying to use a slug that's already
    redirecting to some other instance of the model.  It's easiest to
    do this with functools.partial, e.g.:

        validators=[partial(validate_slug_not_redirecting, 'core', 'Person')]
    """

    model = apps.get_model(app_label, model_name)
    matching_redirects = SlugRedirect.objects.filter(
        content_type=ContentType.objects.get_for_model(model),
        old_object_slug=slug
    )
    if matching_redirects:
        message = 'There is already slug redirection in place for {0}: {1}'
        raise ValidationError(
            message.format(slug, matching_redirects[0])
        )



import re

from django.contrib import admin
from django.db import models
from django.core.exceptions import ValidationError
from .models import SlugRedirect

def stricter_validate_slug(slug):
    if not re.match(r'^[-a-z0-9_]+$', slug):
        raise ValidationError(
            "Enter a valid 'slug' consisting of only lowercase letters, numbers, underscores or hyphens.")
    return True


class StricterSlugFieldMixin(object):
    formfield_overrides = {
        models.SlugField: {
            'validators': [stricter_validate_slug]
        }
    }


@admin.register(SlugRedirect)
class SlugRedirectAdmin(admin.ModelAdmin):
    list_display = [ 'old_object_slug', 'content_type', 'new_object']
    list_filter = ['content_type']
    ordering = ('content_type', 'old_object_slug')



from setuptools import setup, find_packages
setup(
    name = "django-slug-helpers",
    version = "0.0.3",
    packages = find_packages(),
    author = "Mark Longair",
    author_email = "mark-pypi@longair.net",
    description = "A simple model for redirecting from old slugs",
    license = "AGPL",
    keywords = "django slugs",
    install_requires = [
        'Django>=1.8',
    ]
)
